from config import * 
import numpy as np
import matplotlib.pyplot as plt

def compare(beta_impurity, m_impurity, beta_disometry, m_disometry):
    fig = plt.figure(figsize=(8, 6))

    m_exact = exact(beta_impurity)
    plt.plot(beta_impurity, np.abs(m_impurity-m_exact), label='impurity tensor')

    m_exact = exact(beta_disometry)
    plt.plot(beta_disometry, np.abs(m_disometry-m_exact), label=r'$\frac{1}{\beta}\frac{\partial \ln Z}{\partial h}$')
    plt.xlabel(r'$\beta$')
    plt.ylabel(r'error')
    plt.legend()

    plt.show()
    plt.savefig('mag_error.pdf')

def exact(beta):
    betac = np.log(1+np.sqrt(2.))/2
    res = []
    for b in beta:
        res.append( 0 if b < betac else (1-np.sinh(2*b)**(-4))**0.125 )
    return np.array(res)

if __name__=='__main__':

    beta_impurity, m_impurity = np.loadtxt('../D20N80.log', unpack=True, comments='#', usecols=(0, 2))
    beta_disometry, m_disometry = np.loadtxt('../D20N80_disometry.log', unpack=True, comments='#', usecols=(0, 2))
 
    compare(beta_impurity, np.abs(m_impurity), beta_disometry, np.abs(m_disometry))
