import torch
from adlib import SVD
svd = SVD.apply

def gauge(T, Dcut, side):
    D = T.shape[0]
    M = torch.einsum('ludr,adcb->laucrb', (T, T)).contiguous()
    if side=='l':
        M = M.view(D**2, -1)
        M = M@M.t()
    elif side=='r':
        M = M.view(-1, D**2)
        M = M.t()@M

    U, S, V = svd(M)
    Dnew = min(D**2, Dcut)
    truncation = S[Dnew:].sum() / S.sum()
    return U[:, :Dnew].view(D, D, Dnew), truncation

def trace(T):
    trace = 0.0 
    for x in range(T.shape[0]):
        for y in range(T.shape[1]):
            trace += T[x, y, y, x]
    return trace



def build_tensor(K):
    lam = [torch.cosh(K)*2, torch.sinh(K)*2]
    T = []
    for i in range(2):
        for j in range(2):
            for k in range(2):
                for l in range(2):
                    if ((i+j+k+l)%2==0):
                        T.append(torch.sqrt(lam[i]*lam[j]*lam[k]*lam[l])/2.)
                    else:
                        T.append(torch.tensor(0.0, dtype=K.dtype, device=K.device))
    T = torch.stack(T).view(2, 2, 2, 2)
    return T

def expm_naive(A, tol=1E-13):
    eA = torch.eye(A.shape[0], dtype=A.dtype, device=A.device)
    trm = torch.eye(A.shape[0], dtype=A.dtype, device=A.device)
    k = 1
    while (torch.norm(trm)>tol):
        trm = trm@A / k
        eA += trm
        k += 1
    return eA

if __name__=='__main__':
    A = torch.randn(10, 10, dtype=torch.float64)

    import scipy.linalg 
    res_scipy = scipy.linalg.expm(A.numpy())
    res_scipy = torch.from_numpy(res_scipy)

    res_naive = expm_naive(A)

    from expm import expm
    res1 = expm(A)

    from expm2 import expm
    res2 = expm(A)
    print ((res_naive-res_scipy).abs().max())
    print ((res1-res_scipy).abs().max())
    print ((res2-res_scipy).abs().max())




